import { logger } from "./logger.js";
import { TimerError } from "./error.js";
import { getActiveTimers, getTimer, deleteTimer as repoDeleteTimer } from "./timer.repository.js";
import { logTimer } from "./timer-screen.js";
import { TimerCompleteScreen } from "./timer-complete-screen.js";

const INTERVAL_MS = 1000; // main loop interval

const state = {
  timerInterval: null,
  subscribers: [],
};

export function getStatus() {
  return state.timerInterval === null;
}

export function flipTimers() {
  if (getStatus()) {
    startTimers();
  } else {
    stopTimers();
  }
}

export function startTimers() {
  if (state.timerInterval !== null) {
    throw new TimerError('Loop already running');
  }

  logger.debug(`Starting main timer loop`);
  state.timerInterval = setInterval(() => runLoop(), INTERVAL_MS);
  ping();
}

export function subscribe(cb) {
  state.subscribers.push(cb);
}

export function stopTimers() {
  if (state.timerInterval === null) {
    throw new TimerError('Loop not running.');
  }

  logger.debug('Stopping main timer loop.');
  clearInterval(state.timerInterval);
  state.timerInterval = null;
  ping();
}

export function pauseTimer(timerName = '') {
  const timer = getTimer(timerName);
  if (timer.isPaused) {
    logger.error(`Timer ${timerName} already paused.`);
  } else {
    logger.info(`Pausing timer ${timerName}.`);
    timer.isPaused = true;
    ping();
  }
}

export function resumeTimer(timerName = '') {
  const timer = getTimer(timerName);
  if (!timer.isPaused) {
    logger.error(`Timer ${timerName} already running.`);
  } else {
    logger.info(`Resuming timer ${timerName}.`);
    timer.isPaused = false;
    ping();
  }
}

export function deleteTimer(timerName) {
  repoDeleteTimer(timerName);
}

function runLoop() {
  logger.debug(`Running loop...`);
  const timers = getActiveTimers();
  logger.debug(`Got ${timers.length} timers.`);

  for (const timer of timers) {
    advance(timer, INTERVAL_MS);
    logTimer(timer);
  }
  if (timers.length > 0) {
    ping();
  }
}

function ping() {
  for (const sub of state.subscribers) {
    sub();
  }
}

function advance(timer, interval_ms) {
  logger.group('int');
  logger.log(timer);
  logger.log(`Updating interval, timer before: ${timer.elapsedTimeSeconds}, interval: ${interval_ms}`)
  timer.elapsedTimeSeconds += (interval_ms / 1000);
  logger.log(`Interval after: ${timer.elapsedTimeSeconds}`);
  logger.groupEnd('int');
  if (Math.round(timer.elapsedTimeSeconds) >= timer.durationSeconds) {
    completeTimer(timer);
  }
}

// DINGDING
function completeTimer(timer) {
  logger.log(`Timer ${timer.name} completed.`);
  timer.isPaused = true;
  timer.isComplete = true;
  // ping? close something?
  const timerComplete = new TimerCompleteScreen();
  timerComplete.setData({ name: timer.name });
  timerComplete.render(true);
}
